@@ -47,7 +47,7 @@ final class InjectionRequestProcessor extends AbstractProcessor {
   @Override
   public Boolean visit(StaticInjectionRequest request) {
     staticInjections.add(new StaticInjection(injector, request));
-    injector.state.putStaticInjectionRequest(request);
+    injector.getBindingData().putStaticInjectionRequest(request);
     return true;
   }
 
@@ -68,11 +68,13 @@ final class InjectionRequestProcessor extends AbstractProcessor {
     // when determining the types for members injection.
     // If/when this is fixed, we can report the exact type back to the user.
     // (Otherwise the injection points exposed from the request may be wrong.)
-    injector.state.putInjectionRequest(
-        new InjectionRequest<>(
-            request.getSource(),
-            TypeLiteral.get(request.getInstance().getClass()),
-            /* instance= */ null));
+    injector
+        .getBindingData()
+        .putInjectionRequest(
+            new InjectionRequest<>(
+                request.getSource(),
+                TypeLiteral.get(request.getInstance().getClass()),
+                /* instance= */ null));
     return true;
   }
 
