@@ -16,7 +16,6 @@
 
 package com.google.inject;
 
-import static com.google.inject.Asserts.asModuleChain;
 import static com.google.inject.Asserts.assertContains;
 import static com.google.inject.Asserts.assertNotSerializable;
 import static com.google.inject.Asserts.getDeclaringSourcePart;
@@ -125,31 +124,23 @@ public class BinderTest extends TestCase {
       fail("Expected CreationException");
     } catch (CreationException e) {
       assertEquals(4, e.getErrorMessages().size());
-      String segment1 = "No implementation for java.lang.Runnable was bound.";
-      String segment2 = "No implementation for " + Comparator.class.getName() + " was bound.";
-      String segment3 =
-          "No implementation for java.util.concurrent.Callable<java.lang.String> was" + " bound.";
+      String segment1 = "No implementation for Runnable was bound.";
+      String segment2 = "No implementation for Comparator was bound.";
+      String segment3 = "No implementation for Callable<String> was bound.";
       String segment4 =
-          "No implementation for java.util.Date annotated with @"
-              + Named.class.getName()
-              + "(value="
-              + Annotations.memberValueString("date")
-              + ") was bound.";
-      String atSegment = "at " + getClass().getName();
+          String.format(
+              "No implementation for Date annotated with @Named(%s) was bound.",
+              Annotations.memberValueString("value", "date"));
       String sourceFileName = getDeclaringSourcePart(getClass());
       assertContains(
           e.getMessage(),
           segment1,
-          atSegment,
           sourceFileName,
           segment2,
-          atSegment,
           sourceFileName,
           segment3,
-          atSegment,
           sourceFileName,
           segment4,
-          atSegment,
           sourceFileName);
     }
   }
@@ -168,11 +159,10 @@ public class BinderTest extends TestCase {
       assertEquals(1, e.getErrorMessages().size());
       assertContains(
           e.getMessage(),
-          "No implementation for java.lang.Runnable was bound.",
-          "for field at " + NeedsRunnable.class.getName(),
-          ".runnable(BinderTest.java:",
-          "at " + getClass().getName(),
-          getDeclaringSourcePart(getClass()));
+          "No implementation for Runnable was bound.",
+          ".runnable",
+          "for field runnable",
+          "at BinderTest$7.configure");
     }
   }
 
@@ -193,8 +183,8 @@ public class BinderTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) Missing constant value. Please call to(...).",
-          "at " + getClass().getName());
+          "Missing constant value. Please call to(...).",
+          "at BinderTest$8.configure");
     }
   }
 
@@ -210,10 +200,7 @@ public class BinderTest extends TestCase {
       fail();
     } catch (CreationException expected) {
       assertContains(
-          expected.getMessage(),
-          "1) Binding points to itself.",
-          "at " + getClass().getName(),
-          getDeclaringSourcePart(getClass()));
+          expected.getMessage(), "Binding points to itself.", "at BinderTest$9.configure");
     }
   }
 
@@ -329,9 +316,9 @@ public class BinderTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) A binding to java.lang.String[] was already configured at " + getClass().getName(),
-          "at " + getClass().getName(),
-          getDeclaringSourcePart(getClass()));
+          "String[] was bound multiple times.",
+          "1  : BinderTest$18.configure",
+          "2  : BinderTest$18.configure");
       assertContains(expected.getMessage(), "1 error");
     }
 
@@ -393,12 +380,9 @@ public class BinderTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) A binding to java.lang.String was already configured at "
-              + ConstantModule.class.getName(),
-          asModuleChain(ParentModule.class, FooModule.class, ConstantModule.class),
-          "at " + ConstantModule.class.getName(),
-          getDeclaringSourcePart(getClass()),
-          asModuleChain(ParentModule.class, BarModule.class, ConstantModule.class));
+          "String was bound multiple times.",
+          " BinderTest$ParentModule -> BinderTest$FooModule -> BinderTest$ConstantModule",
+          " BinderTest$ParentModule -> BinderTest$BarModule -> BinderTest$ConstantModule");
       assertContains(expected.getMessage(), "1 error");
     }
   }
@@ -419,12 +403,9 @@ public class BinderTest extends TestCase {
       expected.printStackTrace();
       assertContains(
           expected.getMessage(),
-          "1) A binding to "
-              + HasImplementedBy1.class.getName()
-              + " was already configured at "
-              + getClass().getName(),
-          "at " + getClass().getName(),
-          getDeclaringSourcePart(getClass()));
+          "BinderTest$HasImplementedBy1 was bound multiple times.",
+          "1  : BinderTest$22.configure",
+          "2  : BinderTest$HasImplementedBy1.class");
       assertContains(expected.getMessage(), "1 error");
     }
   }
@@ -492,13 +473,9 @@ public class BinderTest extends TestCase {
     } catch (ConfigurationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) No implementation for "
-              + NoInjectConstructor.class.getName()
-              + " (with no qualifier annotation) was bound, and could not find an injectable"
-              + " constructor",
-          "for the 1st parameter of "
-              + MissingParameter.class.getName()
-              + ".<init>(BinderTest.java:");
+          "No injectable constructor for type BinderTest$NoInjectConstructor.",
+          "at BinderTest$MissingParameter.<init>",
+          "for 1st parameter noInjectConstructor");
     }
   }
 
@@ -550,9 +527,8 @@ public class BinderTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) Binding to Provider is not allowed.",
-          "at " + BinderTest.class.getName(),
-          getDeclaringSourcePart(getClass()));
+          "Binding to Provider is not allowed.",
+          "at BinderTest$28.configure");
     }
   }
 
@@ -568,17 +544,17 @@ public class BinderTest extends TestCase {
 
     @Override
     protected void configure() {
-      bind(AbstractModule.class).annotatedWith(red).toProvider(Providers.<AbstractModule>of(null));
-      bind(Binder.class).annotatedWith(red).toProvider(Providers.<Binder>of(null));
-      bind(Binding.class).annotatedWith(red).toProvider(Providers.<Binding>of(null));
-      bind(Injector.class).annotatedWith(red).toProvider(Providers.<Injector>of(null));
-      bind(Key.class).annotatedWith(red).toProvider(Providers.<Key>of(null));
-      bind(Module.class).annotatedWith(red).toProvider(Providers.<Module>of(null));
-      bind(Provider.class).annotatedWith(red).toProvider(Providers.<Provider>of(null));
-      bind(Scope.class).annotatedWith(red).toProvider(Providers.<Scope>of(null));
-      bind(Stage.class).annotatedWith(red).toProvider(Providers.<Stage>of(null));
-      bind(TypeLiteral.class).annotatedWith(red).toProvider(Providers.<TypeLiteral>of(null));
-      bind(new TypeLiteral<Key<String>>() {}).toProvider(Providers.<Key<String>>of(null));
+      bind(AbstractModule.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(Binder.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(Binding.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(Injector.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(Key.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(Module.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(Provider.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(Scope.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(Stage.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(TypeLiteral.class).annotatedWith(red).toProvider(Providers.of(null));
+      bind(new TypeLiteral<Key<String>>() {}).toProvider(Providers.of(null));
     }
   }
 
@@ -587,41 +563,43 @@ public class BinderTest extends TestCase {
       Guice.createInjector(new OuterCoreModule());
       fail();
     } catch (CreationException expected) {
+      String methodLocation = "at BinderTest$InnerCoreModule.configure";
+      String moduleChain = "installed by: BinderTest$OuterCoreModule -> BinderTest$InnerCoreModule";
       assertContains(
           expected.getMessage(),
           "Binding to core guice framework type is not allowed: AbstractModule.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to core guice framework type is not allowed: Binder.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to core guice framework type is not allowed: Binding.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to core guice framework type is not allowed: Injector.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to core guice framework type is not allowed: Key.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to core guice framework type is not allowed: Module.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to Provider is not allowed.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to core guice framework type is not allowed: Scope.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to core guice framework type is not allowed: Stage.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to core guice framework type is not allowed: TypeLiteral.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class),
+          methodLocation,
+          moduleChain,
           "Binding to core guice framework type is not allowed: Key.",
-          "at " + InnerCoreModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterCoreModule.class, InnerCoreModule.class));
+          methodLocation,
+          moduleChain);
     }
   }
 
@@ -707,8 +685,8 @@ public class BinderTest extends TestCase {
     } catch (ConfigurationException expected) {
       Asserts.assertContains(
           expected.getMessage(),
-          "1) Cannot inject a Provider that has no type parameter",
-          "while locating " + Provider.class.getName());
+          "Cannot inject a Provider that has no type parameter",
+          "while locating Provider");
     }
   }
 }
