@@ -17,9 +17,9 @@
 package com.google.inject;
 
 import static com.google.inject.Asserts.assertContains;
-import static com.google.inject.Asserts.getDeclaringSourcePart;
 import static java.lang.annotation.RetentionPolicy.RUNTIME;
 
+import com.google.inject.internal.Annotations;
 import java.lang.annotation.Annotation;
 import java.lang.annotation.Retention;
 import junit.framework.TestCase;
@@ -59,10 +59,9 @@ public class BindingAnnotationTest extends TestCase {
       assertContains(
           expected.getMessage(),
           true,
-          "No implementation for java.lang.String annotated with",
-          "BindingAnnotationTest$Blue(value=5) was bound",
-          "at " + BindingAnnotationTest.class.getName(),
-          getDeclaringSourcePart(getClass()));
+          "No implementation for String annotated with",
+          "BindingAnnotationTest$Blue(" + Annotations.memberValueString("value", 5) + ") was bound",
+          "at BindingAnnotationTest$2.configure");
     }
   }
 
@@ -95,10 +94,9 @@ public class BindingAnnotationTest extends TestCase {
       assertContains(
           expected.getMessage(),
           true,
-          "No implementation for java.lang.String annotated with",
+          "No implementation for String annotated with",
           "BindingAnnotationTest$Color",
-          "at " + BindingAnnotationTest.class.getName(),
-          getDeclaringSourcePart(getClass()));
+          "at BindingAnnotationTest$4.configure");
     }
   }
 
@@ -118,10 +116,9 @@ public class BindingAnnotationTest extends TestCase {
       assertContains(
           expected.getMessage(),
           true,
-          "No implementation for java.lang.String annotated with",
-          "BindingAnnotationTest$Blue(value=5) was bound",
-          "at " + BindingAnnotationTest.class.getName(),
-          getDeclaringSourcePart(getClass()));
+          "No implementation for String annotated with",
+          "BindingAnnotationTest$Blue(" + Annotations.memberValueString("value", 5) + ") was bound",
+          "at BindingAnnotationTest$5.configure");
     }
   }
 
