@@ -21,6 +21,7 @@ import static com.google.inject.name.Names.named;
 
 import com.google.common.base.Objects;
 import com.google.common.collect.Lists;
+import com.google.inject.internal.Annotations;
 import com.google.inject.name.Named;
 import com.google.inject.spi.Element;
 import com.google.inject.spi.Elements;
@@ -84,12 +85,9 @@ public class DuplicateBindingsTest extends TestCase {
     } catch (CreationException ce) {
       assertContains(
           ce.getMessage(),
-          "A binding to "
-              + Foo.class.getName()
-              + " was already configured "
-              + "at "
-              + FailingProviderModule.class.getName(),
-          "at " + FailingProviderModule.class.getName());
+          "DuplicateBindingsTest$Foo was bound multiple times.",
+          "1  : DuplicateBindingsTest$FailingProviderModule.foo",
+          "2  : DuplicateBindingsTest$FailingProviderModule.foo");
     }
   }
 
@@ -128,33 +126,15 @@ public class DuplicateBindingsTest extends TestCase {
           new ScopedModule(Scopes.SINGLETON, foo, pFoo, pclFoo, clFoo, cFoo));
       fail("expected exception");
     } catch (CreationException ce) {
+      String template =
+          "DuplicateBindingsTest$Foo annotated with @Named(%s)" + " was bound multiple times.";
       String segment1 =
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("pInstance")
-              + " was already configured at "
-              + SimpleModule.class.getName();
-      String segment2 =
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("pKey")
-              + " was already configured at "
-              + SimpleModule.class.getName();
+          String.format(template, Annotations.memberValueString("value", "pInstance"));
+      String segment2 = String.format(template, Annotations.memberValueString("value", "pKey"));
       String segment3 =
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("constructor")
-              + " was already configured at "
-              + SimpleModule.class.getName();
-      String segment4 =
-          "A binding to "
-              + FooImpl.class.getName()
-              + " was already configured at "
-              + SimpleModule.class.getName();
-      String atSegment = "at " + ScopedModule.class.getName();
+          String.format(template, Annotations.memberValueString("value", "constructor"));
+      String segment4 = "DuplicateBindingsTest$FooImpl was bound multiple times";
+      String atSegment = "DuplicateBindingsTest$ScopedModule.configure";
       if (isIncludeStackTraceOff()) {
         assertContains(
             ce.getMessage(),
@@ -194,36 +174,33 @@ public class DuplicateBindingsTest extends TestCase {
               (Constructor) Bar.cxtor()));
       fail("expected exception");
     } catch (CreationException ce) {
+      String template =
+          "DuplicateBindingsTest$Foo annotated with @Named(%s) was bound multiple times.";
+      String segment1 = String.format(template, Annotations.memberValueString("value", "instance"));
+      String segment2 =
+          String.format(template, Annotations.memberValueString("value", "pInstance"));
+      String segment3 = String.format(template, Annotations.memberValueString("value", "pKey"));
+      String segment4 =
+          String.format(template, Annotations.memberValueString("value", "linkedKey"));
+      String segment5 =
+          String.format(template, Annotations.memberValueString("value", "constructor"));
       assertContains(
           ce.getMessage(),
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("pInstance")
-              + " was already configured at "
-              + SimpleModule.class.getName(),
-          "at " + SimpleModule.class.getName(),
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("pKey")
-              + " was already configured at "
-              + SimpleModule.class.getName(),
-          "at " + SimpleModule.class.getName(),
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("linkedKey")
-              + " was already configured at "
-              + SimpleModule.class.getName(),
-          "at " + SimpleModule.class.getName(),
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("constructor")
-              + " was already configured at "
-              + SimpleModule.class.getName(),
-          "at " + SimpleModule.class.getName());
+          segment1,
+          "1  : DuplicateBindingsTest$SimpleModule.configure",
+          "2  : DuplicateBindingsTest$SimpleModule.configure",
+          segment2,
+          "1  : DuplicateBindingsTest$SimpleModule.configure",
+          "2  : DuplicateBindingsTest$SimpleModule.configure",
+          segment3,
+          "1  : DuplicateBindingsTest$SimpleModule.configure",
+          "2  : DuplicateBindingsTest$SimpleModule.configure",
+          segment4,
+          "1  : DuplicateBindingsTest$SimpleModule.configure",
+          "2  : DuplicateBindingsTest$SimpleModule.configure",
+          segment5,
+          "1  : DuplicateBindingsTest$SimpleModule.configure",
+          "2  : DuplicateBindingsTest$SimpleModule.configure");
     }
   }
 
@@ -234,12 +211,11 @@ public class DuplicateBindingsTest extends TestCase {
     } catch (CreationException ce) {
       assertContains(
           ce.getMessage(),
-          "A binding to "
-              + Foo.class.getName()
-              + " was already configured at "
-              + ThrowingModule.class.getName(),
-          "and an error was thrown while checking duplicate bindings.  Error: java.lang.RuntimeException: Boo!",
-          "at " + ThrowingModule.class.getName());
+          "A binding to DuplicateBindingsTest$Foo was already configured at"
+              + " DuplicateBindingsTest$ThrowingModule.configure",
+          "and an error was thrown while checking duplicate bindings.  Error:"
+              + " RuntimeException: Boo!",
+          "at DuplicateBindingsTest$ThrowingModule.configure");
     }
   }
 
@@ -250,43 +226,22 @@ public class DuplicateBindingsTest extends TestCase {
       injector.createChildInjector(new SimpleModule(foo, pFoo, pclFoo, clFoo, cFoo));
       fail("expected exception");
     } catch (CreationException ce) {
+      String template =
+          "DuplicateBindingsTest$Foo annotated with @Named(%s) was bound" + " multiple times.";
+      String atSegment = "DuplicateBindingsTest$SimpleModule.configure";
       assertContains(
           ce.getMessage(),
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("pInstance")
-              + " was already configured at "
-              + SimpleModule.class.getName(),
-          "at " + SimpleModule.class.getName(),
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("pKey")
-              + " was already configured at "
-              + SimpleModule.class.getName(),
-          "at " + SimpleModule.class.getName(),
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("linkedKey")
-              + " was already configured at "
-              + SimpleModule.class.getName(),
-          "at " + SimpleModule.class.getName(),
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("constructor")
-              + " was already configured at "
-              + SimpleModule.class.getName(),
-          "at " + SimpleModule.class.getName(),
-          "A binding to "
-              + Foo.class.getName()
-              + " annotated with "
-              + named("providerMethod")
-              + " was already configured at "
-              + SimpleProviderModule.class.getName(),
-          "at " + SimpleProviderModule.class.getName());
+          String.format(template, Annotations.memberValueString("value", "instance")),
+          atSegment,
+          String.format(template, Annotations.memberValueString("value", "pKey")),
+          atSegment,
+          String.format(template, Annotations.memberValueString("value", "linkedKey")),
+          atSegment,
+          String.format(template, Annotations.memberValueString("value", "constructor")),
+          atSegment,
+          String.format(template, Annotations.memberValueString("value", "providerMethod")),
+          "1  : DuplicateBindingsTest$SimpleProviderModule.foo",
+          "2  : DuplicateBindingsTest$SimpleProviderModule.foo");
     }
   }
 
@@ -314,11 +269,9 @@ public class DuplicateBindingsTest extends TestCase {
           if (e1 != e2) {
             assertContains(
                 expected.getMessage(),
-                "A binding to "
-                    + Foo.class.getName()
-                    + " was already configured at "
-                    + FailedModule.class.getName(),
-                "at " + FailedModule.class.getName());
+                "DuplicateBindingsTest$Foo was bound multiple times.",
+                "1  : DuplicateBindingsTest$FailedModule.",
+                "2  : DuplicateBindingsTest$FailedModule.");
           } else {
             throw expected;
           }
@@ -504,7 +457,10 @@ public class DuplicateBindingsTest extends TestCase {
     }
 
     @Override
-    @SuppressWarnings({"EqualsHashCode", "EqualsBrokenForNull"}) // intentionally NPE on null for the test
+    @SuppressWarnings({
+      "EqualsHashCode",
+      "EqualsBrokenForNull"
+    }) // intentionally NPE on null for the test
     public boolean equals(Object obj) {
       return obj.getClass() == getClass();
     }
