@@ -16,9 +16,7 @@
 
 package com.google.inject;
 
-import static com.google.inject.Asserts.asModuleChain;
 import static com.google.inject.Asserts.assertContains;
-import static com.google.inject.Asserts.getDeclaringSourcePart;
 import static com.google.inject.name.Names.named;
 import static java.lang.annotation.RetentionPolicy.RUNTIME;
 
@@ -161,12 +159,13 @@ public class ScopesTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          A.class.getName() + " is annotated with " + Singleton.class.getName(),
-          "but scope annotations are not supported for abstract types.",
-          "at " + A.class.getName() + ".class(ScopesTest.java:");
+          "ScopesTest$A is annotated with Singleton, but scope annotations are not supported for"
+              + " abstract types.",
+          "at ScopesTest$A.class");
     }
   }
 
+  @SuppressWarnings("InjectScopeAnnotationOnInterfaceOrAbstractClass") // for testing
   @Singleton
   interface A {}
 
@@ -175,6 +174,7 @@ public class ScopesTest extends TestCase {
   @Retention(RUNTIME)
   @interface Component {}
 
+  @SuppressWarnings("InjectScopeAnnotationOnInterfaceOrAbstractClass") // for testing
   @Component
   @Singleton
   interface ComponentAnnotationTest {}
@@ -198,12 +198,13 @@ public class ScopesTest extends TestCase {
     } catch (ConfigurationException expected) {
       assertContains(
           expected.getMessage(),
-          D.class.getName() + " is annotated with " + Singleton.class.getName(),
-          "but scope annotations are not supported for abstract types.",
-          "at " + D.class.getName() + ".class(ScopesTest.java:");
+          "ScopesTest$D is annotated with Singleton, but scope annotations are not supported for"
+              + " abstract types.",
+          "at ScopesTest$D.class");
     }
   }
 
+  @SuppressWarnings("InjectScopeAnnotationOnInterfaceOrAbstractClass") // for testing
   @Singleton
   @ImplementedBy(DImpl.class)
   interface D {}
@@ -217,12 +218,13 @@ public class ScopesTest extends TestCase {
     } catch (ConfigurationException expected) {
       assertContains(
           expected.getMessage(),
-          E.class.getName() + " is annotated with " + Singleton.class.getName(),
-          "but scope annotations are not supported for abstract types.",
-          "at " + E.class.getName() + ".class(ScopesTest.java:");
+          "ScopesTest$E is annotated with Singleton, but scope annotations are not supported for"
+              + " abstract types.",
+          "at ScopesTest$E.class");
     }
   }
 
+  @SuppressWarnings("InjectScopeAnnotationOnInterfaceOrAbstractClass") // for testing
   @Singleton
   @ProvidedBy(EProvider.class)
   interface E {}
@@ -248,11 +250,9 @@ public class ScopesTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) No scope is bound to " + CustomScoped.class.getName(),
-          "at " + getClass().getName(),
-          getDeclaringSourcePart(getClass()),
-          "2) No scope is bound to " + CustomScoped.class.getName(),
-          "at " + C.class.getName() + ".class");
+          "No scope is bound to ScopesTest$CustomScoped.",
+          "1  : ScopesTest$5.configure",
+          "2  : ScopesTest$C.class");
     }
   }
 
@@ -333,11 +333,9 @@ public class ScopesTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) Please annotate "
-              + NotRuntimeRetainedScoped.class.getName()
-              + " with @Retention(RUNTIME).",
-          "at " + InnerRuntimeModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterRuntimeModule.class, InnerRuntimeModule.class));
+          "Please annotate ScopesTest$NotRuntimeRetainedScoped with @Retention(RUNTIME).",
+          "at ScopesTest$InnerRuntimeModule.configure",
+          "ScopesTest$OuterRuntimeModule -> ScopesTest$InnerRuntimeModule");
     }
   }
 
@@ -362,9 +360,9 @@ public class ScopesTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) Please annotate " + Deprecated.class.getName() + " with @ScopeAnnotation.",
-          "at " + InnerDeprecatedModule.class.getName() + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterDeprecatedModule.class, InnerDeprecatedModule.class));
+          "Please annotate Deprecated with @ScopeAnnotation.",
+          "at ScopesTest$InnerDeprecatedModule.configure",
+          "installed by: ScopesTest$OuterDeprecatedModule -> ScopesTest$InnerDeprecatedModule");
     }
   }
 
@@ -397,16 +395,11 @@ public class ScopesTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) Scope Scopes.NO_SCOPE is already bound to "
-              + CustomScoped.class.getName()
-              + " at "
-              + CustomNoScopeModule.class.getName()
-              + getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterScopeModule.class, CustomNoScopeModule.class),
+          "Scope Scopes.NO_SCOPE is already bound to ScopesTest$CustomScoped",
+          "ScopesTest$OuterScopeModule -> ScopesTest$CustomNoScopeModule",
           "Cannot bind Scopes.SINGLETON.",
-          "at " + ScopesTest.class.getName(),
-          getDeclaringSourcePart(getClass()),
-          asModuleChain(OuterScopeModule.class, CustomSingletonModule.class));
+          "at ScopesTest$CustomSingletonModule.configure",
+          "ScopesTest$OuterScopeModule -> ScopesTest$CustomSingletonModule");
     }
   }
 
@@ -442,8 +435,8 @@ public class ScopesTest extends TestCase {
     } catch (ConfigurationException expected) {
       assertContains(
           expected.getMessage(),
-          "1) More than one scope annotation was found: ",
-          "while locating " + SingletonAndCustomScoped.class.getName());
+          "More than one scope annotation was found: Singleton and ScopesTest$CustomScoped.",
+          "while locating ScopesTest$SingletonAndCustomScoped");
     }
   }
 
@@ -505,6 +498,7 @@ public class ScopesTest extends TestCase {
         }
       };
 
+  @SuppressWarnings("InjectScopeOrQualifierAnnotationRetention") // to check failure mode
   @Target({ElementType.TYPE, ElementType.METHOD})
   @ScopeAnnotation
   public @interface NotRuntimeRetainedScoped {}
