@@ -66,7 +66,7 @@ public class TypeLiteralInjectionTest extends TestCase {
   }
 
   public void testInjectTypeLiteralWithRawTypes() {
-    C c = Guice.createInjector().getInstance(C.class);
+    C<?> c = Guice.createInjector().getInstance(C.class);
     assertEquals(TypeLiteral.get(String.class), c.string);
     assertEquals(TypeLiteral.get(A.class), c.a);
 
@@ -76,9 +76,7 @@ public class TypeLiteralInjectionTest extends TestCase {
     } catch (ConfigurationException expected) {
       assertContains(
           expected.getMessage(),
-          TypeLiteral.class.getName()
-              + "<java.util.List<T>> "
-              + "cannot be used as a key; It is not fully specified.");
+          "TypeLiteral<List<T>> cannot be used as a key; It is not fully specified.");
     }
   }
 
@@ -110,6 +108,7 @@ public class TypeLiteralInjectionTest extends TestCase {
     @Inject TypeLiteral<T> t;
   }
 
+  @SuppressWarnings("rawtypes") // Testing rawtypes.
   static class C<T> {
     @Inject TypeLiteral<String> string;
     @Inject TypeLiteral<A> a;
