@@ -24,6 +24,7 @@ import static com.google.inject.internal.SpiUtils.linked;
 import static com.google.inject.internal.SpiUtils.providerInstance;
 import static com.google.inject.internal.SpiUtils.providerKey;
 import static com.google.inject.name.Names.named;
+import static org.junit.Assert.assertThrows;
 
 import com.google.common.base.Optional;
 import com.google.common.collect.ImmutableSet;
@@ -107,7 +108,6 @@ public class OptionalBinderTest extends TestCase {
             requireBinding(new Key<Optional<String>>() {}); // the above specifies this.
             requireBinding(String.class); // but it doesn't specify this.
             binder().requireExplicitBindings(); // need to do this, otherwise String will JIT
-
             requireBinding(Key.get(javaOptionalOfString));
           }
         };
@@ -117,12 +117,47 @@ public class OptionalBinderTest extends TestCase {
       fail();
     } catch (CreationException ce) {
       assertContains(
-          ce.getMessage(),
-          "1) Explicit bindings are required and java.lang.String is not explicitly bound.");
+          ce.getMessage(), "Explicit bindings are required and String is not explicitly bound.");
       assertEquals(1, ce.getErrorMessages().size());
     }
   }
 
+  public void testLinkedTypeSameAsBaseType() {
+    Module module =
+        new AbstractModule() {
+          @Override
+          protected void configure() {
+            OptionalBinder.newOptionalBinder(binder(), MyClass.class)
+                .setBinding()
+                .to(MyClass.class);
+          }
+        };
+
+    CreationException ce =
+        assertThrows(CreationException.class, () -> Guice.createInjector(module));
+    assertContains(ce.getMessage(), "Binding points to itself. Key: OptionalBinderTest$MyClass");
+  }
+
+  public void testLinkedAndBaseTypeHaveDifferentAnnotations() {
+    Module module =
+        new AbstractModule() {
+          @Override
+          protected void configure() {
+            OptionalBinder.newOptionalBinder(binder(), Key.get(MyClass.class, Names.named("foo")))
+                .setBinding()
+                .to(Key.get(MyClass.class, Names.named("moo")));
+          }
+
+          @Provides
+          @Named("moo")
+          MyClass provideString() {
+            return new MyClass();
+          }
+        };
+    Injector injector = Guice.createInjector(module);
+    assertNotNull(injector);
+  }
+
   public void testOptionalIsAbsentByDefault() throws Exception {
     Module module =
         new AbstractModule() {
@@ -604,12 +639,7 @@ public class OptionalBinderTest extends TestCase {
       assertEquals(ce.getMessage(), 1, ce.getErrorMessages().size());
       assertContains(
           ce.getMessage(),
-          "1) A binding to java.lang.String annotated with @"
-              + RealOptionalBinder.Default.class.getName()
-              + " was already configured at "
-              + module.getClass().getName()
-              + ".configure(",
-          "at " + module.getClass().getName() + ".configure(");
+          "String annotated with @RealOptionalBinder$Default was bound multiple times.");
     }
   }
 
@@ -629,12 +659,9 @@ public class OptionalBinderTest extends TestCase {
       assertEquals(ce.getMessage(), 1, ce.getErrorMessages().size());
       assertContains(
           ce.getMessage(),
-          "1) A binding to java.lang.String annotated with @"
-              + RealOptionalBinder.Actual.class.getName()
-              + " was already configured at "
-              + module.getClass().getName()
-              + ".configure(",
-          "at " + module.getClass().getName() + ".configure(");
+          "String annotated with @RealOptionalBinder$Actual was bound multiple times.",
+          "1  : " + getShortName(module) + ".configure",
+          "2  : " + getShortName(module) + ".configure");
     }
   }
 
@@ -656,18 +683,8 @@ public class OptionalBinderTest extends TestCase {
       assertEquals(ce.getMessage(), 2, ce.getErrorMessages().size());
       assertContains(
           ce.getMessage(),
-          "1) A binding to java.lang.String annotated with @"
-              + RealOptionalBinder.Default.class.getName()
-              + " was already configured at "
-              + module.getClass().getName()
-              + ".configure(",
-          "at " + module.getClass().getName() + ".configure(",
-          "2) A binding to java.lang.String annotated with @"
-              + RealOptionalBinder.Actual.class.getName()
-              + " was already configured at "
-              + module.getClass().getName()
-              + ".configure(",
-          "at " + module.getClass().getName() + ".configure(");
+          "String annotated with @RealOptionalBinder$Default was bound multiple times.",
+          "String annotated with @RealOptionalBinder$Actual was bound multiple times.");
     }
   }
 
@@ -974,20 +991,21 @@ public class OptionalBinderTest extends TestCase {
   }
 
   public void testSourceLinesInException() {
+    Module module =
+        new AbstractModule() {
+          @Override
+          protected void configure() {
+            OptionalBinder.newOptionalBinder(binder(), Integer.class).setDefault();
+          }
+        };
     try {
-      Guice.createInjector(
-          new AbstractModule() {
-            @Override
-            protected void configure() {
-              OptionalBinder.newOptionalBinder(binder(), Integer.class).setDefault();
-            }
-          });
+      Guice.createInjector(module);
       fail();
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "No implementation for java.lang.Integer",
-          "at " + getClass().getName());
+          "No implementation for Integer",
+          getShortName(module) + ".configure");
     }
   }
 
@@ -1297,9 +1315,9 @@ public class OptionalBinderTest extends TestCase {
       Key<?> bindingKey = entry.getKey();
       Key<?> clonedKey;
       if (bindingKey.getAnnotation() != null) {
-        clonedKey = Key.get(bindingKey.getTypeLiteral(), bindingKey.getAnnotation());
+        clonedKey = bindingKey.ofType(bindingKey.getTypeLiteral());
       } else if (bindingKey.getAnnotationType() != null) {
-        clonedKey = Key.get(bindingKey.getTypeLiteral(), bindingKey.getAnnotationType());
+        clonedKey = bindingKey.ofType(bindingKey.getTypeLiteral());
       } else {
         clonedKey = Key.get(bindingKey.getTypeLiteral());
       }
@@ -1385,7 +1403,7 @@ public class OptionalBinderTest extends TestCase {
                 bind(String.class).toInstance("hi");
               }
             });
-    WeakKeySetUtils.assertNotBlacklisted(parentInjector, Key.get(Integer.class));
+    WeakKeySetUtils.assertNotBanned(parentInjector, Key.get(Integer.class));
 
     Injector childInjector =
         parentInjector.createChildInjector(
@@ -1398,13 +1416,13 @@ public class OptionalBinderTest extends TestCase {
               }
             });
     WeakReference<Injector> weakRef = new WeakReference<>(childInjector);
-    WeakKeySetUtils.assertBlacklisted(parentInjector, Key.get(Integer.class));
+    WeakKeySetUtils.assertBanned(parentInjector, Key.get(Integer.class));
 
-    // Clear the ref, GC, and ensure that we are no longer blacklisting.
+    // Clear the ref, GC, and ensure that we are no longer banning.
     childInjector = null;
 
     Asserts.awaitClear(weakRef);
-    WeakKeySetUtils.assertNotBlacklisted(parentInjector, Key.get(Integer.class));
+    WeakKeySetUtils.assertNotBanned(parentInjector, Key.get(Integer.class));
   }
 
   public void testCompareEqualsAgainstOtherAnnotation() {
@@ -1422,6 +1440,15 @@ public class OptionalBinderTest extends TestCase {
     assertFalse(other1.equals(other2));
   }
 
+  /**
+   * Returns the short name for a module instance. Used to get the name of the anoymous class that
+   * can change depending on the order the module intance is created.
+   */
+  private static String getShortName(Module module) {
+    String fullName = module.getClass().getName();
+    return fullName.substring(fullName.lastIndexOf(".") + 1);
+  }
+
   @RealOptionalBinder.Actual("foo")
   @RealOptionalBinder.Default("foo")
   static class Dummy {}
@@ -1430,4 +1457,6 @@ public class OptionalBinderTest extends TestCase {
   private <V> Set<V> setOf(V... elements) {
     return ImmutableSet.copyOf(elements);
   }
+
+  static class MyClass {}
 }
