@@ -18,9 +18,9 @@ package com.google.inject.internal;
 
 import static com.google.inject.Asserts.awaitClear;
 import static com.google.inject.Asserts.awaitFullGc;
-import static com.google.inject.internal.WeakKeySetUtils.assertBlacklisted;
+import static com.google.inject.internal.WeakKeySetUtils.assertBanned;
 import static com.google.inject.internal.WeakKeySetUtils.assertInSet;
-import static com.google.inject.internal.WeakKeySetUtils.assertNotBlacklisted;
+import static com.google.inject.internal.WeakKeySetUtils.assertNotBanned;
 import static com.google.inject.internal.WeakKeySetUtils.assertNotInSet;
 import static com.google.inject.internal.WeakKeySetUtils.assertSourceNotInSet;
 
@@ -35,6 +35,7 @@ import com.google.inject.Key;
 import com.google.inject.Scope;
 import com.google.inject.TypeLiteral;
 import com.google.inject.spi.InjectionRequest;
+import com.google.inject.spi.InterceptorBinding;
 import com.google.inject.spi.MembersInjectorLookup;
 import com.google.inject.spi.ModuleAnnotatedMethodScannerBinding;
 import com.google.inject.spi.ProviderLookup;
@@ -47,6 +48,7 @@ import java.lang.annotation.Annotation;
 import java.lang.ref.WeakReference;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 import junit.framework.TestCase;
 
@@ -67,16 +69,16 @@ public class WeakKeySetTest extends TestCase {
   }
 
   public void testEviction() {
-    TestState state = new TestState();
+    TestInjectorBindingData bindingData = new TestInjectorBindingData();
     Key<Integer> key = Key.get(Integer.class);
     Object source = new Object();
 
     WeakReference<Key<Integer>> weakKeyRef = new WeakReference<>(key);
 
-    set.add(key, state, source);
+    set.add(key, bindingData, source);
     assertInSet(set, key, 1, source);
 
-    state = null;
+    bindingData = null;
 
     awaitFullGc();
 
@@ -88,16 +90,16 @@ public class WeakKeySetTest extends TestCase {
   }
 
   public void testEviction_nullSource() {
-    TestState state = new TestState();
+    TestInjectorBindingData bindingData = new TestInjectorBindingData();
     Key<Integer> key = Key.get(Integer.class);
     Object source = null;
 
     WeakReference<Key<Integer>> weakKeyRef = new WeakReference<>(key);
 
-    set.add(key, state, source);
+    set.add(key, bindingData, source);
     assertInSet(set, key, 1, source);
 
-    state = null;
+    bindingData = null;
 
     awaitFullGc();
 
@@ -109,17 +111,17 @@ public class WeakKeySetTest extends TestCase {
   }
 
   public void testEviction_keyOverlap_2x() {
-    TestState state1 = new TestState();
-    TestState state2 = new TestState();
+    TestInjectorBindingData bindingData1 = new TestInjectorBindingData();
+    TestInjectorBindingData bindingData2 = new TestInjectorBindingData();
     Key<Integer> key1 = Key.get(Integer.class);
     Key<Integer> key2 = Key.get(Integer.class);
     Object source1 = new Object();
     Object source2 = new Object();
 
-    set.add(key1, state1, source1);
+    set.add(key1, bindingData1, source1);
     assertInSet(set, key1, 1, source1);
 
-    set.add(key2, state2, source2);
+    set.add(key2, bindingData2, source2);
     assertInSet(set, key2, 2, source1, source2);
 
     WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<>(key1);
@@ -128,23 +130,24 @@ public class WeakKeySetTest extends TestCase {
     WeakReference<Object> weakSource2Ref = new WeakReference<>(source2);
 
     Key<Integer> key = key1 = key2 = Key.get(Integer.class);
-    state1 = null;
+    bindingData1 = null;
 
     awaitFullGc();
 
     assertSourceNotInSet(set, key, source1);
     assertInSet(set, key, 1, source2);
 
-    source1 = source2 = null;
+    // Clear source1 and source2 fields so the objects can be GCed.
+    Object unused = source1 = source2 = null;
 
     awaitClear(weakSource1Ref);
     // Key1 will be referenced as the key in the sources backingSet and won't be
     // GC'd.
 
-    // Should not be GC'd until state2 goes away.
+    // Should not be GC'd until bindingData2 goes away.
     assertNotNull(weakSource2Ref.get());
 
-    state2 = null;
+    bindingData2 = null;
 
     awaitFullGc();
 
@@ -157,17 +160,17 @@ public class WeakKeySetTest extends TestCase {
   }
 
   public void testNoEviction_keyOverlap_2x() {
-    TestState state1 = new TestState();
-    TestState state2 = new TestState();
+    TestInjectorBindingData bindingData1 = new TestInjectorBindingData();
+    TestInjectorBindingData bindingData2 = new TestInjectorBindingData();
     Key<Integer> key1 = Key.get(Integer.class);
     Key<Integer> key2 = Key.get(Integer.class);
     Object source1 = new Object();
     Object source2 = new Object();
 
-    set.add(key1, state1, source1);
+    set.add(key1, bindingData1, source1);
     assertInSet(set, key1, 1, source1);
 
-    set.add(key2, state2, source2);
+    set.add(key2, bindingData2, source2);
     assertInSet(set, key2, 2, source1, source2);
 
     WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<>(key1);
@@ -178,23 +181,24 @@ public class WeakKeySetTest extends TestCase {
     awaitFullGc();
     assertInSet(set, key, 2, source1, source2);
 
-    // Ensure the keys don't get GC'd when states are still referenced. key1 will be present in the
+    // Ensure the keys don't get GC'd when InjectorBindingData objects are still referenced. key1
+    // will be present in the
     // as the map key but key2 could be GC'd if the implementation does something wrong.
     assertNotNull(weakKey1Ref.get());
     assertNotNull(weakKey2Ref.get());
   }
 
   public void testEviction_keyAndSourceOverlap_null() {
-    TestState state1 = new TestState();
-    TestState state2 = new TestState();
+    TestInjectorBindingData bindingData1 = new TestInjectorBindingData();
+    TestInjectorBindingData bindingData2 = new TestInjectorBindingData();
     Key<Integer> key1 = Key.get(Integer.class);
     Key<Integer> key2 = Key.get(Integer.class);
     Object source = null;
 
-    set.add(key1, state1, source);
+    set.add(key1, bindingData1, source);
     assertInSet(set, key1, 1, source);
 
-    set.add(key2, state2, source);
+    set.add(key2, bindingData2, source);
     // Same source so still only one value.
     assertInSet(set, key2, 1, source);
     assertInSet(set, key1, 1, source);
@@ -204,7 +208,7 @@ public class WeakKeySetTest extends TestCase {
     WeakReference<Object> weakSourceRef = new WeakReference<>(source);
 
     Key<Integer> key = key1 = key2 = Key.get(Integer.class);
-    state1 = null;
+    bindingData1 = null;
 
     awaitFullGc();
     // Should still have a single source.
@@ -216,7 +220,7 @@ public class WeakKeySetTest extends TestCase {
     // Key1 will be referenced as the key in the sources backingSet and won't be
     // GC'd.
 
-    state2 = null;
+    bindingData2 = null;
 
     awaitFullGc();
     assertNotInSet(set, key);
@@ -228,16 +232,16 @@ public class WeakKeySetTest extends TestCase {
   }
 
   public void testEviction_keyAndSourceOverlap_nonNull() {
-    TestState state1 = new TestState();
-    TestState state2 = new TestState();
+    TestInjectorBindingData bindingData1 = new TestInjectorBindingData();
+    TestInjectorBindingData bindingData2 = new TestInjectorBindingData();
     Key<Integer> key1 = Key.get(Integer.class);
     Key<Integer> key2 = Key.get(Integer.class);
     Object source = new Object();
 
-    set.add(key1, state1, source);
+    set.add(key1, bindingData1, source);
     assertInSet(set, key1, 1, source);
 
-    set.add(key2, state2, source);
+    set.add(key2, bindingData2, source);
     // Same source so still only one value.
     assertInSet(set, key2, 1, source);
 
@@ -246,7 +250,7 @@ public class WeakKeySetTest extends TestCase {
     WeakReference<Object> weakSourceRef = new WeakReference<>(source);
 
     Key<Integer> key = key1 = key2 = Key.get(Integer.class);
-    state1 = null;
+    bindingData1 = null;
 
     awaitFullGc();
 
@@ -261,7 +265,7 @@ public class WeakKeySetTest extends TestCase {
     // Key1 will be referenced as the key in the sources backingSet and won't be
     // GC'd.
 
-    state2 = null;
+    bindingData2 = null;
 
     awaitFullGc();
 
@@ -274,9 +278,9 @@ public class WeakKeySetTest extends TestCase {
   }
 
   public void testEviction_keyOverlap_3x() {
-    TestState state1 = new TestState();
-    TestState state2 = new TestState();
-    TestState state3 = new TestState();
+    TestInjectorBindingData bindingData1 = new TestInjectorBindingData();
+    TestInjectorBindingData bindingData2 = new TestInjectorBindingData();
+    TestInjectorBindingData bindingData3 = new TestInjectorBindingData();
     Key<Integer> key1 = Key.get(Integer.class);
     Key<Integer> key2 = Key.get(Integer.class);
     Key<Integer> key3 = Key.get(Integer.class);
@@ -284,13 +288,13 @@ public class WeakKeySetTest extends TestCase {
     Object source2 = new Object();
     Object source3 = new Object();
 
-    set.add(key1, state1, source1);
+    set.add(key1, bindingData1, source1);
     assertInSet(set, key1, 1, source1);
 
-    set.add(key2, state2, source2);
+    set.add(key2, bindingData2, source2);
     assertInSet(set, key1, 2, source1, source2);
 
-    set.add(key3, state3, source3);
+    set.add(key3, bindingData3, source3);
     assertInSet(set, key1, 3, source1, source2, source3);
 
     WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<>(key1);
@@ -301,7 +305,7 @@ public class WeakKeySetTest extends TestCase {
     WeakReference<Object> weakSource3Ref = new WeakReference<>(source3);
 
     Key<Integer> key = key1 = key2 = key3 = Key.get(Integer.class);
-    state1 = null;
+    bindingData1 = null;
 
     awaitFullGc();
     assertSourceNotInSet(set, key, source1);
@@ -312,7 +316,7 @@ public class WeakKeySetTest extends TestCase {
     // GC'd.
     awaitClear(weakSource1Ref);
 
-    state2 = null;
+    bindingData2 = null;
     awaitFullGc();
     assertSourceNotInSet(set, key, source2);
     assertInSet(set, key, 1, source3);
@@ -324,7 +328,7 @@ public class WeakKeySetTest extends TestCase {
     // Key1 will be referenced as the key in the sources backingSet and won't be
     // GC'd.
 
-    state3 = null;
+    bindingData3 = null;
     awaitFullGc();
     assertNotInSet(set, key);
 
@@ -344,7 +348,7 @@ public class WeakKeySetTest extends TestCase {
                 bind(Integer.class).toInstance(4);
               }
             });
-    assertNotBlacklisted(parentInjector, Key.get(String.class));
+    assertNotBanned(parentInjector, Key.get(String.class));
 
     Injector childInjector =
         parentInjector.createChildInjector(
@@ -355,12 +359,12 @@ public class WeakKeySetTest extends TestCase {
               }
             });
     WeakReference<Injector> weakRef = new WeakReference<>(childInjector);
-    assertBlacklisted(parentInjector, Key.get(String.class));
+    assertBanned(parentInjector, Key.get(String.class));
 
-    // Clear the ref, GC, and ensure that we are no longer blacklisting.
+    // Clear the ref, GC, and ensure that we are no longer banning.
     childInjector = null;
     awaitClear(weakRef);
-    assertNotBlacklisted(parentInjector, Key.get(String.class));
+    assertNotBanned(parentInjector, Key.get(String.class));
   }
 
   public void testWeakKeySet_integration_multipleChildren() {
@@ -372,8 +376,8 @@ public class WeakKeySetTest extends TestCase {
                 bind(Integer.class).toInstance(4);
               }
             });
-    assertNotBlacklisted(parentInjector, Key.get(String.class));
-    assertNotBlacklisted(parentInjector, Key.get(Long.class));
+    assertNotBanned(parentInjector, Key.get(String.class));
+    assertNotBanned(parentInjector, Key.get(Long.class));
 
     Injector childInjector1 =
         parentInjector.createChildInjector(
@@ -384,8 +388,8 @@ public class WeakKeySetTest extends TestCase {
               }
             });
     WeakReference<Injector> weakRef1 = new WeakReference<>(childInjector1);
-    assertBlacklisted(parentInjector, Key.get(String.class));
-    assertNotBlacklisted(parentInjector, Key.get(Long.class));
+    assertBanned(parentInjector, Key.get(String.class));
+    assertNotBanned(parentInjector, Key.get(Long.class));
 
     Injector childInjector2 =
         parentInjector.createChildInjector(
@@ -396,20 +400,20 @@ public class WeakKeySetTest extends TestCase {
               }
             });
     WeakReference<Injector> weakRef2 = new WeakReference<>(childInjector2);
-    assertBlacklisted(parentInjector, Key.get(String.class));
-    assertBlacklisted(parentInjector, Key.get(Long.class));
+    assertBanned(parentInjector, Key.get(String.class));
+    assertBanned(parentInjector, Key.get(Long.class));
 
-    // Clear ref1, GC, and ensure that we still blacklist.
+    // Clear ref1, GC, and ensure that we still ban.
     childInjector1 = null;
     awaitClear(weakRef1);
-    assertNotBlacklisted(parentInjector, Key.get(String.class));
-    assertBlacklisted(parentInjector, Key.get(Long.class));
+    assertNotBanned(parentInjector, Key.get(String.class));
+    assertBanned(parentInjector, Key.get(Long.class));
 
-    // Clear the ref, GC, and ensure that we are no longer blacklisting.
+    // Clear the ref, GC, and ensure that we are no longer banning.
     childInjector2 = null;
     awaitClear(weakRef2);
-    assertNotBlacklisted(parentInjector, Key.get(String.class));
-    assertNotBlacklisted(parentInjector, Key.get(Long.class));
+    assertNotBanned(parentInjector, Key.get(String.class));
+    assertNotBanned(parentInjector, Key.get(Long.class));
   }
 
   public void testWeakKeySet_integration_multipleChildren_overlappingKeys() {
@@ -421,7 +425,7 @@ public class WeakKeySetTest extends TestCase {
                 bind(Integer.class).toInstance(4);
               }
             });
-    assertNotBlacklisted(parentInjector, Key.get(String.class));
+    assertNotBanned(parentInjector, Key.get(String.class));
 
     Injector childInjector1 =
         parentInjector.createChildInjector(
@@ -432,7 +436,7 @@ public class WeakKeySetTest extends TestCase {
               }
             });
     WeakReference<Injector> weakRef1 = new WeakReference<>(childInjector1);
-    assertBlacklisted(parentInjector, Key.get(String.class));
+    assertBanned(parentInjector, Key.get(String.class));
 
     Injector childInjector2 =
         parentInjector.createChildInjector(
@@ -443,23 +447,27 @@ public class WeakKeySetTest extends TestCase {
               }
             });
     WeakReference<Injector> weakRef2 = new WeakReference<>(childInjector2);
-    assertBlacklisted(parentInjector, Key.get(String.class));
+    assertBanned(parentInjector, Key.get(String.class));
 
-    // Clear ref1, GC, and ensure that we still blacklist.
+    // Clear ref1, GC, and ensure that we still ban.
     childInjector1 = null;
     awaitClear(weakRef1);
-    assertBlacklisted(parentInjector, Key.get(String.class));
+    assertBanned(parentInjector, Key.get(String.class));
 
-    // Clear the ref, GC, and ensure that we are no longer blacklisting.
+    // Clear the ref, GC, and ensure that we are no longer banning.
     childInjector2 = null;
     awaitClear(weakRef2);
-    assertNotBlacklisted(parentInjector, Key.get(String.class));
+    assertNotBanned(parentInjector, Key.get(String.class));
   }
 
-  private static class TestState implements State {
+  private static class TestInjectorBindingData extends InjectorBindingData {
+    TestInjectorBindingData() {
+      super(Optional.empty());
+    }
+
     @Override
-    public State parent() {
-      return new TestState();
+    public Optional<InjectorBindingData> parent() {
+      return Optional.of(new TestInjectorBindingData());
     }
 
     @Override
@@ -543,17 +551,15 @@ public class WeakKeySetTest extends TestCase {
       return ImmutableSet.of();
     }
 
-    /*if[AOP]*/
     @Override
-    public void addMethodAspect(MethodAspect methodAspect) {
+    public void addInterceptorBinding(InterceptorBinding interceptorBinding) {
       throw new UnsupportedOperationException();
     }
 
     @Override
-    public ImmutableList<MethodAspect> getMethodAspects() {
+    public ImmutableList<InterceptorBinding> getInterceptorBindings() {
       return ImmutableList.of();
     }
-    /*end[AOP]*/
 
     @Override
     public void addTypeListener(TypeListenerBinding typeListenerBinding) {
@@ -561,7 +567,7 @@ public class WeakKeySetTest extends TestCase {
     }
 
     @Override
-    public List<TypeListenerBinding> getTypeListenerBindings() {
+    public ImmutableList<TypeListenerBinding> getTypeListenerBindings() {
       return ImmutableList.of();
     }
 
@@ -571,7 +577,7 @@ public class WeakKeySetTest extends TestCase {
     }
 
     @Override
-    public List<ProvisionListenerBinding> getProvisionListenerBindings() {
+    public ImmutableList<ProvisionListenerBinding> getProvisionListenerBindings() {
       return ImmutableList.of();
     }
 
@@ -581,28 +587,10 @@ public class WeakKeySetTest extends TestCase {
     }
 
     @Override
-    public List<ModuleAnnotatedMethodScannerBinding> getScannerBindings() {
+    public ImmutableList<ModuleAnnotatedMethodScannerBinding> getScannerBindings() {
       return ImmutableList.of();
     }
 
-    @Override
-    public void blacklist(Key<?> key, State state, Object source) {}
-
-    @Override
-    public boolean isBlacklisted(Key<?> key) {
-      return true;
-    }
-
-    @Override
-    public Set<Object> getSourcesForBlacklistedKey(Key<?> key) {
-      throw new UnsupportedOperationException();
-    }
-
-    @Override
-    public Object lock() {
-      throw new UnsupportedOperationException();
-    }
-
     @Override
     public Map<Class<? extends Annotation>, Scope> getScopes() {
       return ImmutableMap.of();
@@ -614,17 +602,17 @@ public class WeakKeySetTest extends TestCase {
     }
 
     @Override
-    public List<TypeListenerBinding> getTypeListenerBindingsThisLevel() {
+    public ImmutableList<TypeListenerBinding> getTypeListenerBindingsThisLevel() {
       throw new UnsupportedOperationException();
     }
 
     @Override
-    public List<ProvisionListenerBinding> getProvisionListenerBindingsThisLevel() {
+    public ImmutableList<ProvisionListenerBinding> getProvisionListenerBindingsThisLevel() {
       throw new UnsupportedOperationException();
     }
 
     @Override
-    public List<ModuleAnnotatedMethodScannerBinding> getScannerBindingsThisLevel() {
+    public ImmutableList<ModuleAnnotatedMethodScannerBinding> getScannerBindingsThisLevel() {
       throw new UnsupportedOperationException();
     }
   }
