@@ -29,7 +29,6 @@ import com.google.inject.Injector;
 import com.google.inject.Key;
 import com.google.inject.Provider;
 import com.google.inject.TypeLiteral;
-import com.google.inject.internal.BytecodeGen;
 import com.google.inject.internal.Errors;
 import com.google.inject.internal.ErrorsException;
 import com.google.inject.spi.Dependency;
@@ -206,7 +205,7 @@ public class FactoryProvider<F> implements Provider<F>, HasDependencies {
         throw new ConfigurationException(e.getErrors().getMessages());
       }
 
-      return new FactoryProvider2<F>(Key.get(factoryType), collector);
+      return new FactoryProvider2<F>(Key.get(factoryType), collector, /* userLookups= */ null);
     }
   }
 
@@ -326,9 +325,9 @@ public class FactoryProvider<F> implements Provider<F>, HasDependencies {
         for (Annotation parameterAnnotation : parameterAnnotations) {
           if (parameterAnnotation.annotationType() == Assisted.class) {
             throw newConfigurationException(
-                "Factory method %s has an @Assisted parameter, which "
-                    + "is incompatible with the deprecated @AssistedInject annotation. Please replace "
-                    + "@AssistedInject with @Inject on the %s constructor.",
+                "Factory method %s has an @Assisted parameter, which is incompatible with the"
+                    + " deprecated @AssistedInject annotation. Please replace @AssistedInject with"
+                    + " @Inject on the %s constructor.",
                 method, implementationType);
           }
         }
@@ -402,9 +401,7 @@ public class FactoryProvider<F> implements Provider<F>, HasDependencies {
     Class<F> factoryRawType = (Class<F>) (Class<?>) factoryType.getRawType();
     return factoryRawType.cast(
         Proxy.newProxyInstance(
-            BytecodeGen.getClassLoader(factoryRawType),
-            new Class[] {factoryRawType},
-            invocationHandler));
+            factoryRawType.getClassLoader(), new Class<?>[] {factoryRawType}, invocationHandler));
   }
 
   @Override
