@@ -42,7 +42,6 @@ import com.google.inject.ProvisionException;
 import com.google.inject.Scopes;
 import com.google.inject.TypeLiteral;
 import com.google.inject.internal.Annotations;
-import com.google.inject.internal.BytecodeGen;
 import com.google.inject.internal.Errors;
 import com.google.inject.internal.ErrorsException;
 import com.google.inject.internal.UniqueAnnotations;
@@ -59,7 +58,9 @@ import com.google.inject.util.Providers;
 import java.lang.annotation.Annotation;
 import java.lang.invoke.MethodHandle;
 import java.lang.invoke.MethodHandles;
+import java.lang.invoke.MethodType;
 import java.lang.reflect.Constructor;
+import java.lang.reflect.Field;
 import java.lang.reflect.InvocationHandler;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
@@ -71,6 +72,7 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.function.Supplier;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
@@ -95,6 +97,20 @@ final class FactoryProvider2<F>
   // use the logger under a well-known name, not FactoryProvider2
   static final Logger logger = Logger.getLogger(AssistedInject.class.getName());
 
+  /**
+   * A constant that determines if we allow fallback to using the JDK internals to make a "private
+   * lookup". Typically always true, but reflectively set to false in tests.
+   */
+  @SuppressWarnings("FieldCanBeFinal") // non-final for testing
+  private static boolean allowPrivateLookupFallback = true;
+
+  /**
+   * A constant that determines if we allow fallback to using method handle workarounds (if
+   * required). Typically always true, but reflectively set to false in tests.
+   */
+  @SuppressWarnings("FieldCanBeFinal") // non-final for testing
+  private static boolean allowMethodHandleWorkaround = true;
+
   /** if a factory method parameter isn't annotated, it gets this annotation. */
   static final Assisted DEFAULT_ANNOTATION =
       new Assisted() {
@@ -122,8 +138,8 @@ final class FactoryProvider2<F>
         public String toString() {
           return "@"
               + Assisted.class.getName()
-              + "(value="
-              + Annotations.memberValueString("")
+              + "("
+              + Annotations.memberValueString("value", "")
               + ")";
         }
       };
@@ -227,8 +243,10 @@ final class FactoryProvider2<F>
   /**
    * @param factoryKey a key for a Java interface that defines one or more create methods.
    * @param collector binding configuration that maps method return types to implementation types.
+   * @param userLookups user provided lookups, optional.
    */
-  FactoryProvider2(Key<F> factoryKey, BindingCollector collector) {
+  FactoryProvider2(
+      Key<F> factoryKey, BindingCollector collector, MethodHandles.Lookup userLookups) {
     this.factoryKey = factoryKey;
     this.collector = collector;
 
@@ -355,37 +373,87 @@ final class FactoryProvider2<F>
       factory =
           factoryRawType.cast(
               Proxy.newProxyInstance(
-                  BytecodeGen.getClassLoader(factoryRawType),
-                  new Class<?>[] {factoryRawType},
-                  this));
+                  factoryRawType.getClassLoader(), new Class<?>[] {factoryRawType}, this));
 
       // Now go back through default methods. Try to use MethodHandles to make things
       // work.  If that doesn't work, fallback to trying to find compatible method
       // signatures.
       Map<Method, AssistData> dataSoFar = assistDataBuilder.build();
       ImmutableMap.Builder<Method, MethodHandle> methodHandleBuilder = ImmutableMap.builder();
+      boolean warnedAboutUserLookups = false;
       for (Map.Entry<String, Method> entry : defaultMethods.entries()) {
+        if (!warnedAboutUserLookups
+            && userLookups == null
+            && !Modifier.isPublic(factory.getClass().getModifiers())) {
+          warnedAboutUserLookups = true;
+          logger.log(
+              Level.WARNING,
+              "AssistedInject factory {0} is non-public and has javac-generated default methods. "
+                  + " Please pass a `MethodHandles.lookup()` with"
+                  + " FactoryModuleBuilder.withLookups when using this factory so that Guice can"
+                  + " properly call the default methods. Guice will try to workaround this, but "
+                  + "it does not always work (depending on the method signatures of the factory).",
+              new Object[] {factoryType});
+        }
+
+        // Note: If the user didn't supply a valid lookup, we always try to fallback to the hacky
+        // signature comparing workaround below.
+        // This is because all these shenanigans are only necessary because we're implementing
+        // AssistedInject through a Proxy. If we were to generate a subclass (which we theoretically
+        // _could_ do), then we wouldn't inadvertantly proxy the javac-generated default methods
+        // too (and end up with a stack overflow from infinite recursion).
+        // As such, we try our hardest to "make things work" requiring requiring extra effort from
+        // the user.
+
         Method defaultMethod = entry.getValue();
-        MethodHandle handle = createMethodHandle(defaultMethod, factory);
+        MethodHandle handle = null;
+        try {
+          handle =
+              superMethodHandle(
+                  SuperMethodSupport.METHOD_LOOKUP, defaultMethod, factory, userLookups);
+        } catch (ReflectiveOperationException e1) {
+          // If the user-specified lookup failed, try again w/ the private lookup hack.
+          // If _that_ doesn't work, try the below workaround.
+          if (allowPrivateLookupFallback
+              && SuperMethodSupport.METHOD_LOOKUP != SuperMethodLookup.PRIVATE_LOOKUP) {
+            try {
+              handle =
+                  superMethodHandle(
+                      SuperMethodLookup.PRIVATE_LOOKUP, defaultMethod, factory, userLookups);
+            } catch (ReflectiveOperationException e2) {
+              // ignored, use below workaround.
+            }
+          }
+        }
+
+        Supplier<String> failureMsg =
+            () ->
+                "Unable to use non-public factory "
+                    + factoryRawType.getName()
+                    + ". Please call"
+                    + " FactoryModuleBuilder.withLookups(MethodHandles.lookup()) (with a"
+                    + " lookups that has access to the factory), or make the factory"
+                    + " public.";
         if (handle != null) {
           methodHandleBuilder.put(defaultMethod, handle);
+        } else if (!allowMethodHandleWorkaround) {
+          errors.addMessage(failureMsg.get());
         } else {
           boolean foundMatch = false;
           for (Method otherMethod : otherMethods.get(defaultMethod.getName())) {
             if (dataSoFar.containsKey(otherMethod) && isCompatible(defaultMethod, otherMethod)) {
               if (foundMatch) {
-                errors.addMessage(
-                    "Generated default method %s with parameters %s is"
-                        + " signature-compatible with more than one non-default method."
-                        + " Unable to create factory. As a workaround, remove the override"
-                        + " so javac stops generating a default method.",
-                    defaultMethod, Arrays.asList(defaultMethod.getParameterTypes()));
+                errors.addMessage(failureMsg.get());
+                break;
               } else {
                 assistDataBuilder.put(defaultMethod, dataSoFar.get(otherMethod));
                 foundMatch = true;
               }
             }
           }
+          // We always expect to find at least one match, because we only deal with javac-generated
+          // default methods. If we ever allow user-specified default methods, this will need to
+          // change.
           if (!foundMatch) {
             throw new IllegalStateException("Can't find method compatible with: " + defaultMethod);
           }
@@ -548,7 +616,7 @@ final class FactoryProvider2<F>
     }
 
     if (!anyAssistedInjectConstructors) {
-      // If none existed, use @Inject.
+      // If none existed, use @Inject or a no-arg constructor.
       try {
         return InjectionPoint.forConstructorOf(implementation);
       } catch (ConfigurationException e) {
@@ -694,7 +762,7 @@ final class FactoryProvider2<F>
    */
   private <T> Key<T> assistKey(Method method, Key<T> key, Errors errors) throws ErrorsException {
     if (key.getAnnotationType() == null) {
-      return Key.get(key.getTypeLiteral(), DEFAULT_ANNOTATION);
+      return key.withAnnotation(DEFAULT_ANNOTATION);
     } else if (key.getAnnotationType() == Assisted.class) {
       return key;
     } else {
@@ -897,36 +965,133 @@ final class FactoryProvider2<F>
     }
   }
 
-  // Note: this isn't a public API, but we need to use it in order to call default methods on (or
-  // with) non-public types.  If it doesn't exist, the code falls back to a less precise check.
-  private static final Constructor<MethodHandles.Lookup> methodHandlesLookupCxtor =
-      findMethodHandlesLookupCxtor();
+  /**
+   * Holder for the appropriate kind of method lookup to use. Due to bugs in Java releases, we have
+   * to evaluate what approach to take at runtime. We do this by emulating the buggy scenarios: can
+   * a lookup access private details that it should be able to see? If not, we fail down to using
+   * full private access. Unfortunately, private access doesn't work in the JDK17+.... but it
+   * shouldn't be necessary there either, because the buggy lookup checks should be fixed.
+   */
+  private static class SuperMethodSupport {
+    private static final SuperMethodLookup METHOD_LOOKUP;
 
-  private static Constructor<MethodHandles.Lookup> findMethodHandlesLookupCxtor() {
-    try {
-      Constructor<MethodHandles.Lookup> cxtor =
-          MethodHandles.Lookup.class.getDeclaredConstructor(Class.class, int.class);
-      cxtor.setAccessible(true);
-      return cxtor;
-    } catch (ReflectiveOperationException ignored) {
-      // Ignore, the code falls back to a less-precise check if we can't create method handles.
-      return null;
+    static {
+      SuperMethodLookup workingLookup = null;
+      try {
+        Class<?> hidden =
+            Class.forName("com.google.inject.assistedinject.internal.LookupTester$Hidden");
+        Method method = hidden.getMethod("method");
+        Field lookupsField = hidden.getEnclosingClass().getDeclaredField("LOOKUP");
+        lookupsField.setAccessible(true);
+        MethodHandles.Lookup lookups = (MethodHandles.Lookup) lookupsField.get(null);
+        for (SuperMethodLookup attempt : SuperMethodLookup.values()) {
+          try {
+            attempt.superMethodHandle(method, lookups);
+            workingLookup = attempt;
+            break;
+          } catch (ReflectiveOperationException ignored) {
+            // Keep looping to find a working lookup
+          }
+        }
+      } catch (ReflectiveOperationException ignored) {
+        // Bail if our internal tests don't exist.
+      }
+      // If everything failed, use the worst option.
+      if (workingLookup == null) {
+        workingLookup = SuperMethodLookup.PRIVATE_LOOKUP;
+      }
+      METHOD_LOOKUP = workingLookup;
     }
   }
 
-  private static MethodHandle createMethodHandle(Method method, Object proxy) {
-    if (methodHandlesLookupCxtor == null) {
-      return null;
-    }
-    Class<?> declaringClass = method.getDeclaringClass();
-    int allModes =
+  private static MethodHandle superMethodHandle(
+      SuperMethodLookup strategy, Method method, Object proxy, MethodHandles.Lookup userLookups)
+      throws ReflectiveOperationException {
+    MethodHandles.Lookup lookup = userLookups == null ? MethodHandles.lookup() : userLookups;
+    MethodHandle handle = strategy.superMethodHandle(method, lookup);
+    return handle != null ? handle.bindTo(proxy) : null;
+  }
+
+  private static enum SuperMethodLookup {
+    UNREFLECT_SPECIAL {
+      @Override
+      MethodHandle superMethodHandle(Method method, MethodHandles.Lookup lookup)
+          throws ReflectiveOperationException {
+        return lookup.unreflectSpecial(method, method.getDeclaringClass());
+      }
+    },
+    FIND_SPECIAL {
+      @Override
+      MethodHandle superMethodHandle(Method method, MethodHandles.Lookup lookup)
+          throws ReflectiveOperationException {
+        Class<?> declaringClass = method.getDeclaringClass();
+        // Before JDK14, unreflectSpecial didn't work in some scenarios.
+        // So we workaround using findSpecial. See: https://bugs.openjdk.java.net/browse/JDK-8209005
+        return lookup.findSpecial(
+            declaringClass,
+            method.getName(),
+            MethodType.methodType(method.getReturnType(), method.getParameterTypes()),
+            declaringClass);
+      }
+    },
+    PRIVATE_LOOKUP {
+      @Override
+      MethodHandle superMethodHandle(Method method, MethodHandles.Lookup unused)
+          throws ReflectiveOperationException {
+        // Even findSpecial fails on JDK8, so we need to manually reflect on private details.
+        // But note that this will fail 100% of the time on JDK17+, which doesn't allow reflection
+        // into the JDK internals.
+        return PrivateLookup.superMethodHandle(method);
+      }
+    };
+
+    abstract MethodHandle superMethodHandle(Method method, MethodHandles.Lookup lookup)
+        throws ReflectiveOperationException;
+  };
+
+  // Note: this isn't a public API, but we need to use it in order to call default methods on (or
+  // with) non-public types. If it doesn't exist, the code falls back to a less precise check.
+  static class PrivateLookup {
+    PrivateLookup() {}
+
+    private static final int ALL_MODES =
         Modifier.PRIVATE | Modifier.STATIC /* package */ | Modifier.PUBLIC | Modifier.PROTECTED;
-    try {
-      MethodHandles.Lookup lookup = methodHandlesLookupCxtor.newInstance(declaringClass, allModes);
-      method.setAccessible(true);
-      return lookup.unreflectSpecial(method, declaringClass).bindTo(proxy);
-    } catch (ReflectiveOperationException roe) {
-      throw new RuntimeException("Unable to access method: " + method, roe);
+
+    private static final Constructor<MethodHandles.Lookup> privateLookupCxtor =
+        findPrivateLookupCxtor();
+
+    private static Constructor<MethodHandles.Lookup> findPrivateLookupCxtor() {
+      try {
+        Constructor<MethodHandles.Lookup> cxtor;
+        try {
+          cxtor = MethodHandles.Lookup.class.getDeclaredConstructor(Class.class, int.class);
+        } catch (NoSuchMethodException ignored) {
+          cxtor =
+              MethodHandles.Lookup.class.getDeclaredConstructor(
+                  Class.class, Class.class, int.class);
+        }
+        cxtor.setAccessible(true);
+        return cxtor;
+      } catch (Exception e) {
+        // Note: we catch Exception because we want to handle InaccessibleObjectException too,
+        // but we target JDK8.
+        // TODO(sameb): When we drop JDK8 support, catch ReflectiveOperation|Security|Inaccessible
+        return null;
+      }
+    }
+
+    static MethodHandle superMethodHandle(Method method) throws ReflectiveOperationException {
+      if (privateLookupCxtor == null) {
+        return null; // fall back to assistDataBuilder workaround
+      }
+      Class<?> declaringClass = method.getDeclaringClass();
+      MethodHandles.Lookup lookup;
+      if (privateLookupCxtor.getParameterCount() == 2) {
+        lookup = privateLookupCxtor.newInstance(declaringClass, ALL_MODES);
+      } else {
+        lookup = privateLookupCxtor.newInstance(declaringClass, null, ALL_MODES);
+      }
+      return lookup.unreflectSpecial(method, declaringClass);
     }
   }
 }
