@@ -637,10 +637,9 @@ public class FactoryProviderTest extends TestCase {
     } catch (CreationException expected) {
       assertContains(
           expected.getMessage(),
-          "Factory method "
-              + AssistedParamsFactory.class.getName()
-              + ".create() has an @Assisted parameter, which "
-              + "is incompatible with the deprecated @AssistedInject annotation.");
+          "Factory method FactoryProviderTest$AssistedParamsFactory.create() has an @Assisted"
+              + " parameter, which is incompatible with the deprecated @AssistedInject"
+              + " annotation.");
     }
   }
 
