@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (C) 2006 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,20 +20,23 @@ import com.google.inject.Binding;
 
 class ManagedBinding implements ManagedBindingMBean {
 
-  final Binding binding;
+  final Binding<?> binding;
 
-  ManagedBinding(Binding binding) {
+  ManagedBinding(Binding<?> binding) {
     this.binding = binding;
   }
 
+  @Override
   public String getSource() {
     return binding.getSource().toString();
   }
 
+  @Override
   public String getKey() {
     return binding.getKey().toString();
   }
 
+  @Override
   public String getProvider() {
     return binding.getProvider().toString();
   }
