@@ -106,7 +106,7 @@ class FilterDefinition implements ProviderWithExtensionVisitor<FilterDefinition>
       return;
     }
 
-    //initialize our filter with the configured context params and servlet context
+    // initialize our filter with the configured context params and servlet context
     filter.init(
         new FilterConfig() {
           @Override
@@ -125,7 +125,7 @@ class FilterDefinition implements ProviderWithExtensionVisitor<FilterDefinition>
           }
 
           @Override
-          public Enumeration getInitParameterNames() {
+          public Enumeration<String> getInitParameterNames() {
             return Iterators.asEnumeration(initParams.keySet().iterator());
           }
         });
