@@ -39,7 +39,7 @@ import javax.servlet.http.HttpSession;
  *
  * @author crazybob@google.com (Bob Lee)
  */
-public class ServletScopes {
+public final class ServletScopes {
 
   private ServletScopes() {}
 
@@ -76,7 +76,7 @@ public class ServletScopes {
           // request is in progress.
           if (null == GuiceFilter.localContext.get()) {
 
-            // NOTE(dhanji): We don't need to synchronize on the scope map
+            // NOTE(user): We don't need to synchronize on the scope map
             // unlike the HTTP request because we're the only ones who have
             // a reference to it, and it is only available via a threadlocal.
             Context context = requestScopeContext.get();
@@ -398,7 +398,7 @@ public class ServletScopes {
   }
 
   private static class Context implements RequestScoper {
-    final Map<Key, Object> map = Maps.newHashMap();
+    final Map<Key<?>, Object> map = Maps.newHashMap();
 
     // Synchronized to prevent two threads from using the same request
     // scope concurrently.
