@@ -17,6 +17,7 @@
 package com.google.inject.testing.fieldbinder;
 
 import static com.google.inject.Asserts.assertContains;
+import static java.lang.annotation.ElementType.TYPE_USE;
 import static java.lang.annotation.RetentionPolicy.RUNTIME;
 
 import com.google.common.collect.Iterables;
@@ -29,11 +30,15 @@ import com.google.inject.Injector;
 import com.google.inject.Key;
 import com.google.inject.Provider;
 import com.google.inject.ProvisionException;
+import com.google.inject.RestrictedBindingSource;
 import com.google.inject.name.Named;
 import com.google.inject.name.Names;
+import com.google.inject.spi.ElementSource;
 import com.google.inject.testing.fieldbinder.BoundFieldModule.BoundFieldInfo;
 import com.google.inject.util.Providers;
 import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
+import java.lang.annotation.Target;
 import java.util.Arrays;
 import java.util.List;
 import javax.inject.Qualifier;
@@ -144,9 +149,7 @@ public class BoundFieldModuleTest extends TestCase {
       injector.getInstance(Integer.class);
       fail();
     } catch (ConfigurationException e) {
-      assertContains(
-          e.getMessage(),
-          "No implementation for java.lang.Integer (with no qualifier annotation) was bound");
+      assertContains(e.getMessage(), "No injectable constructor for type Integer.");
     }
   }
 
@@ -459,14 +462,22 @@ public class BoundFieldModuleTest extends TestCase {
   @Retention(RUNTIME)
   private @interface Nullable {}
 
+  private static class TypeUse {
+    @Retention(RUNTIME)
+    @Target(TYPE_USE)
+    private @interface Nullable {}
+  }
+
   public void testBindingNullableNullField() {
     Object instance =
         new Object() {
           @Bind @Nullable private Integer anInt = null;
+          @Bind @TypeUse.Nullable private Long aLong = null;
         };
 
     Injector injector = Guice.createInjector(BoundFieldModule.of(instance));
     assertNull(injector.getInstance(Integer.class));
+    assertNull(injector.getInstance(Long.class));
   }
 
   public void testBindingNullProvider() {
@@ -547,9 +558,7 @@ public class BoundFieldModuleTest extends TestCase {
       injector.getInstance(Integer.class);
       fail();
     } catch (ConfigurationException e) {
-      assertContains(
-          e.getMessage(),
-          "No implementation for java.lang.Integer (with no qualifier annotation) was bound");
+      assertContains(e.getMessage(), "No injectable constructor for type Integer.");
     }
   }
 
@@ -609,6 +618,7 @@ public class BoundFieldModuleTest extends TestCase {
     assertEquals(testArray, injector.getInstance(Integer[].class));
   }
 
+  @SuppressWarnings("rawtypes") // Testing rawtypes
   public void testRawProviderCannotBeBound() {
     final Integer testValue = 1024;
     Object instance =
@@ -636,6 +646,7 @@ public class BoundFieldModuleTest extends TestCase {
     }
   }
 
+  @SuppressWarnings("rawtypes") // Testing rawtypes
   public void testExplicitlyBoundRawProviderCanBeBound() {
     final Integer testValue = 1024;
     Object instance =
@@ -656,6 +667,7 @@ public class BoundFieldModuleTest extends TestCase {
     assertEquals(testValue, injector.getInstance(Integer.class));
   }
 
+  @SuppressWarnings("rawtypes") // Testing rawtypes
   public void testRawProviderCanBindToIncorrectType() {
     final Integer testValue = 1024;
     Object instance =
@@ -679,7 +691,7 @@ public class BoundFieldModuleTest extends TestCase {
   public void testMultipleBindErrorsAreAggregated() {
     Object instance =
         new Object() {
-          @Bind private Provider aProvider;
+          @Bind private Provider<Object> aProvider;
 
           @Bind(to = String.class)
           private Integer anInt;
@@ -795,7 +807,7 @@ public class BoundFieldModuleTest extends TestCase {
       Guice.createInjector(module);
       fail();
     } catch (CreationException e) {
-      assertContains(e.getMessage(), "at " + InvalidBindableClass.class.getName() + ".anInt");
+      assertContains(e.getMessage(), "at BoundFieldModuleTest$InvalidBindableClass.anInt");
     }
   }
 
@@ -1019,4 +1031,35 @@ public class BoundFieldModuleTest extends TestCase {
 
     assertTrue(info.getBindAnnotation().lazy());
   }
+
+  @Qualifier
+  @RestrictedBindingSource(
+      explanation = "",
+      permits = {TestPermit.class})
+  @Retention(RetentionPolicy.RUNTIME)
+  @interface Foo {}
+
+  public void testBoundFieldModuleWithPermits() {
+    class Bindings {
+      @Bind @Foo int foo = 17;
+    }
+    Bindings bindings = new Bindings();
+
+    Injector injector =
+        Guice.createInjector(new BoundFieldModule.@TestPermit WithPermits(bindings) {});
+
+    assertEquals((Integer) bindings.foo, injector.getInstance(Key.get(Integer.class, Foo.class)));
+  }
+
+  public void testSourceSetOnBinding() throws Exception {
+    Object instance =
+        new Object() {
+          @Bind Integer value = 1;
+        };
+    BoundFieldModule module = BoundFieldModule.of(instance);
+    Injector injector = Guice.createInjector(module);
+    assertEquals(
+        instance.getClass().getDeclaredField("value"),
+        ((ElementSource) injector.getBinding(Integer.class).getSource()).getDeclaringSource());
+  }
 }
